
/*
 * KITNARF's Java Library <http://www.kitnarf.cz/java/>.
 * COPYRIGHT (C) 2009 Petr Frantík (www.kitnarf.cz). ALL RIGHTS RESERVED. 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package cz.kitnarf.priklad3;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;

import cz.kitnarf.color.SeptilinearColorFunction;
import cz.kitnarf.color.GrayColorFunction;

//******************************************************************************
/**
 *
 *@author Petr Frantík
 *@version 20140224
 */
public class Priklad3
{
	//4e-15,5e-14,1.2e-12
	public static double i=3e-15;	//5e-14
	public static double m=0.05;
	public static double t=0.0002;
	public static double ro=7850;
	public static double l=5;

/*	
	public static double i=5e-14;
	public static double m=2;
	public static double t=0.001;
	public static double ro=1000;
	public static double l=5;
/*	


	/** Makes step by direct iteration.*/
	public static void stepDirectIteration(double[] variables)
	{
		double b=variables[0];
		double h=variables[1];
		//
		double ht3=(h-2d*t)*(h-2d*t)*(h-2d*t);
		variables[0]=Math.pow((12d*i+(b-2d*t)*ht3)/b,1d/3d);
		variables[1]=(m/ro/l+(b-2d*t)*(h-2d*t))/h;
		//
		//System.out.println(variables[0]+" "+variables[1]);
	}



	/** Calculates difference vector.*/
	public static double[] differences(double[] variables)
	{
		double b=variables[0];
		double h=variables[1];
		//
		double bt=b-2d*t;
		double ht=h-2d*t;
		//
		double dm=ro*l*(b*h-bt*ht)-m;
		double di=1d/12d*(b*h*h*h-bt*ht*ht*ht)-i;
		//
		//System.out.println(dm+" "+di);
		//
		double[] differences=new double[]{dm,di};
		return differences;
	}



	/** Calculates average of relative differences.*/
	public static double difference(double[] variables)
	{
		double[] differences=differences(variables);
		//
		double rm=Math.abs(differences[0]/m);
		double ri=Math.abs(differences[1]/i);
		//
		//return 0.5d*(ri+rm);
		double vectorLength=Math.sqrt(rm*rm+ri*ri);
		double coef=1;
		if(vectorLength>100) coef=0.001;
		else if(vectorLength>10) coef=0.01;
		else if(vectorLength>1) coef=0.1;
		else if(vectorLength>0.1) coef=1;
		else if(vectorLength>0.01) coef=10;
		else if(vectorLength>0.001) coef=100;
		return vectorLength*coef;
	}




	/** Main.*/
	public static void main(String[] arguments) throws Exception
	{
		int imageWidth=1024;
		double deltaB=0.006;
		double deltaH=0.0005;
		//
		//System.out.println(difference(new double[]{0.001856179,0.000686294}));
		//
		BufferedImage image=new BufferedImage(imageWidth,imageWidth,BufferedImage.TYPE_INT_RGB);
		Graphics2D g2d=image.createGraphics();
		//SeptilinearColorFunction colorFunction=new SeptilinearColorFunction();
		GrayColorFunction colorFunction=new GrayColorFunction();
		//
		double[] variables=new double[2];
		for(int x=0;x<imageWidth;x++)
		{
			for(int y=0;y<imageWidth;y++)
			{
				variables[0]=-deltaB+2*deltaB/(imageWidth-1)*x;	//b
				variables[1]=-deltaH+2*deltaH/(imageWidth-1)*y;	//h
				//
				//for(int i=0;i<1000;i++) stepDirectIteration(variables);
				double difference=difference(variables);
				//
				Color color=Color.BLACK;
				if(difference<1d) color=colorFunction.getSafeColor(difference);
				//
				g2d.setColor(color);
				g2d.drawLine(x,y,x,y);
			}
		}
		//
		ImageIO.write(image,"PNG",new File("result.png"));
	}



}
